enum PackageType {
  hosted,
  git,
  path,
}

abstract class PackageDetails {
  static PackageType getType(Map<String, dynamic> json) {
    if (json.values.first is String) {
      return PackageType.hosted;
    }
    return PackageType.hosted;
  }

  final String name;

  const PackageDetails(this.name);

  factory PackageDetails.fromJson(Map<String, dynamic> json) {
    final type = getType(json);
    if (type == PackageType.hosted) {
      return HostedPackage.fromJson(json);
    }
    throw 'Unknown package type';
  }

  MapEntry<String, dynamic> toMapEntry();
}

class HostedPackage extends PackageDetails {
  final String version;

  const HostedPackage(String name, this.version) : super(name);

  factory HostedPackage.fromJson(Map<String, dynamic> json) =>
      HostedPackage(json.keys.first, json.values.first);

  MapEntry<String, dynamic> toMapEntry() => MapEntry(name, version);
}

class PathPackage extends PackageDetails {
  final String path;

  const PathPackage(String name, this.path) : super(name);

  factory PathPackage.fromJson(Map<String, dynamic> json) {
    print(json);
    return null;
  }

  MapEntry<String, dynamic> toMapEntry() => MapEntry(name, {'path': path});
}
